@@ -28,17 +28,9 @@ import numpy as np
 
 from mdtraj import (DCDTrajectoryFile, BINPOSTrajectoryFile, XTCTrajectoryFile,
                     TRRTrajectoryFile, HDF5TrajectoryFile, NetCDFTrajectoryFile,
-                    PDBTrajectoryFile, Topology)
+                    PDBTrajectoryFile, MDCRDTrajectoryFile, Topology)
 from mdtraj.utils import unitcell, ensure_type
 
-try:
-    from simtk.openmm import Vec3
-    from simtk.unit import nanometer
-    HAVE_OPENMM = True
-except ImportError:
-    HAVE_OPENMM = False
-
-
 __all__ = ['Trajectory', 'load', 'load_pdb', 'load_xtc', 'load_trr', 'load_binpos',
            'load_dcd', 'load_netcdf', 'load_hdf5', 'load_netcdf', 'load_xml']
 
@@ -84,7 +76,7 @@ def _parse_topology(top):
     elif isinstance(top, Topology):
         topology = top
     else:
-        raise TypeError('Could not interpreted top=%s' % top)
+        raise TypeError('A topology is required. You supplied top=%s' % top)
 
     return topology
 
@@ -105,6 +97,31 @@ def _convert(quantity, in_unit, out_unit, inplace=False):
     quantity *= factor
 
 
+def _cast_indices(indices):
+    """Check that ``indices`` are approprate for indexing an array
+
+    Parameters
+    ----------
+    indices : {None, array_like, slice}
+        If indices is None or slice, it'll just pass through. Otherwise, it'll
+        be converted to a numpy array and checked to make sure it contains
+        integers.
+
+    Returns
+    -------
+    value : {slice, np.ndarray}
+        Either a slice or an array of integers, depending on the input type
+    """
+    if indices is None or isinstance(indices, slice):
+        return indices
+
+    out = np.asarray(indices)
+    if not issubclass(out.dtype.type, np.integer):
+        raise ValueError('indices must be of an integer type. %s is not an integer type' % out.dtype)
+
+    return out
+
+
 ##############################################################################
 # Utilities
 ##############################################################################
@@ -211,14 +228,12 @@ def load_pdb(filename, stride=None, atom_indices=None):
         raise TypeError('filename must be of type string for load_pdb. '
             'you supplied %s' % type(filename))
 
-    if atom_indices is None:
-        _atom_slice = slice(None)
-    else:
-        _atom_slice = atom_indices
+    atom_indices = _cast_indices(atom_indices)
 
     filename = str(filename)
     with PDBTrajectoryFile(filename) as f:
-        coords = f.positions[::stride, _atom_slice, :]
+        atom_slice = slice(None) if atom_indices is None else atom_indices
+        coords = f.positions[::stride, atom_slice, :]
         assert coords.ndim == 3, 'internal shape error'
         n_frames = len(coords)
 
@@ -334,6 +349,8 @@ def load_xtc(filename, top=None, stride=None, atom_indices=None):
             'you supplied %s' % type(filename))
 
     topology = _parse_topology(top)
+
+    atom_indices = _cast_indices(atom_indices)
     if atom_indices is not None:
         topology = topology.subset(atom_indices)
 
@@ -388,6 +405,7 @@ def load_trr(filename, top=None, stride=None, atom_indices=None):
             'you supplied %s' % type(filename))
 
     topology = _parse_topology(top)
+    atom_indices = _cast_indices(atom_indices)
     if atom_indices is not None:
         topology = topology.subset(atom_indices)
 
@@ -443,6 +461,7 @@ def load_dcd(filename, top=None, stride=None, atom_indices=None):
             'you supplied %s' % type(filename))
 
     topology = _parse_topology(top)
+    atom_indices = _cast_indices(atom_indices)
     if atom_indices is not None:
         topology = topology.subset(atom_indices)
 
@@ -491,6 +510,7 @@ def load_hdf5(filename, stride=None, atom_indices=None, frame=None):
     --------
     mdtraj.HDF5TrajectoryFile :  Low level interface to HDF5 files
     """
+    atom_indices = _cast_indices(atom_indices)
 
     with HDF5TrajectoryFile(filename) as f:
         if frame is not None:
@@ -551,8 +571,8 @@ def load_binpos(filename, top=None, stride=None, atom_indices=None):
         raise TypeError('filename must be of type string for load_binpos. '
             'you supplied %s' % type(filename))
 
-
     topology = _parse_topology(top)
+    atom_indices = _cast_indices(atom_indices)
     if atom_indices is not None:
         topology = topology.subset(atom_indices)
 
@@ -571,6 +591,71 @@ def load_binpos(filename, top=None, stride=None, atom_indices=None):
     return Trajectory(xyz=xyz, topology=topology, time=time)
 
 
+def load_mdcrd(filename, top=None, stride=None, atom_indices=None):
+    """Load an AMBER mdcrd file.
+
+    Parameters
+    ----------
+    filename : str
+        String filename of AMBER mdcrd file.
+    top : {str, Trajectory, Topology}
+        The BINPOS format does not contain topology information. Pass in either
+        the path to a pdb file, a trajectory, or a topology to supply this
+        information.
+    stride : int, default=None
+        Only read every stride-th frame
+    atom_indices : array_like, optional
+        If not none, then read only a subset of the atoms coordinates from the
+        file.
+
+    Returns
+    -------
+    trajectory : md.Trajectory
+        The resulting trajectory, as an md.Trajectory object.
+
+    See Also
+    --------
+    mdtraj.MDCRDTrajectoryFile :  Low level interface to MDCRD files
+    """
+    # we make it not required in the signature, but required here. although this
+    # is a little wierd, its good because this function is usually called by a
+    # dispatch from load(), where top comes from **kwargs. So if its not supplied
+    # we want to give the user an informative error message
+    if top is None:
+        raise ValueError('"top" argument is required for load_mdcrd')
+
+    if not isinstance(filename, basestring):
+        raise TypeError('filename must be of type string for load_mdcrd. '
+            'you supplied %s' % type(filename))
+
+    topology = _parse_topology(top)
+    atom_indices = _cast_indices(atom_indices)
+    if atom_indices is not None:
+        topology = topology.subset(atom_indices)
+
+    with MDCRDTrajectoryFile(filename, n_atoms=top._numAtoms) as f:
+        xyz, cell_lengths = f.read(stride=stride, atom_indices=atom_indices)
+        _convert(xyz, f.distance_unit, Trajectory._distance_unit, inplace=True)
+
+        if cell_lengths is not None:
+            _convert(cell_lengths, f.distance_unit, Trajectory._distance_unit, inplace=True)
+
+            # Assume that its a rectilinear box
+            cell_angles = 90.0 * np.ones_like(cell_lengths)
+
+    time = np.arange(len(xyz))
+    if stride is not None:
+        # if we loaded with a stride, the Trajectories's time field should
+        # respect that
+        time *= stride
+
+    t = Trajectory(xyz=xyz, topology=topology, time=time)
+    if cell_lengths is not None:
+        t.unitcell_lengths = cell_lengths
+        t.unitcell_angles = cell_angles
+    return t
+
+
 def load_netcdf(filename, top=None, stride=None, atom_indices=None):
     """Load an AMBER NetCDF file. Since the NetCDF format doesn't contain
     information to specify the topolgy, you need to supply a topology
@@ -600,6 +685,7 @@ def load_netcdf(filename, top=None, stride=None, atom_indices=None):
     mdtraj.NetCDFTrajectoryFile :  Low level interface to NetCDF files
     """
     topology = _parse_topology(top)
+    atom_indices = _cast_indices(atom_indices)
     if atom_indices is not None:
         topology = topology.subset(atom_indices)
 
@@ -1081,9 +1167,8 @@ class Trajectory(object):
             for input to OpenMM
 
         """
-        # copied from Lee-Ping Wang's Molecule.py
-        if not HAVE_OPENMM:
-            raise ImportError('OpenMM was not imported')
+        from simtk.openmm import Vec3
+        from simtk.unit import nanometer
 
         Pos = []
         for xyzi in self.xyz[frame]:
@@ -1110,9 +1195,8 @@ class Trajectory(object):
             The periodic box vectors for this frame, formatted for input to
             OpenMM.
         """
-        # copied from Lee-Ping Wang's Molecule.py
-        if not HAVE_OPENMM:
-            raise ImportError('OpenMM was not imported')
+        from simtk.openmm import Vec3
+        from simtk.unit import nanometer
 
         vectors = self[frame].unitcell_vectors
         if vectors is None:
@@ -1160,13 +1244,15 @@ class Trajectory(object):
         extension = os.path.splitext(filename)[1]
 
         savers = {'.xtc': self.save_xtc,
-          '.trr': self.save_trr,
-          '.pdb': self.save_pdb,
-          '.dcd': self.save_dcd,
-          '.h5': self.save_hdf5,
-          '.binpos': self.save_binpos,
-          '.nc': self.save_netcdf,
-          '.ncdf': self.save_netcdf}
+                  '.trr': self.save_trr,
+                  '.pdb': self.save_pdb,
+                  '.dcd': self.save_dcd,
+                  '.h5': self.save_hdf5,
+                  '.binpos': self.save_binpos,
+                  '.nc': self.save_netcdf,
+                  '.crd': self.save_mdcrd,
+                  '.mdcrd': self.save_mdcrd,
+                  '.ncdf': self.save_netcdf}
 
         try:
             saver = savers[extension]
@@ -1282,6 +1368,26 @@ class Trajectory(object):
             f.write(_convert(self.xyz, Trajectory._distance_unit, f.distance_unit))
 
 
+    def save_mdcrd(self, filename, force_overwrite=True):
+        """Save trajectory to AMBER mdcrd format
+
+        Parameters
+        ----------
+        filename : str
+            filesystem path in which to save the trajectory
+        force_overwrite : bool, default=True
+            Overwrite anything that exists at filename, if its already there
+        """
+        self._check_valid_unitcell()
+        if self._have_unitcell:
+            if not np.all(self.unitcell_angles == 90):
+                raise ValueError('Only rectilinear boxes can be saved to mdcrd files')
+
+        with MDCRDTrajectoryFile(filename, mode='w', force_overwrite=force_overwrite) as f:
+            f.write(_convert(self.xyz, Trajectory._distance_unit, f.distance_unit),
+                    _convert(self.unitcell_lengths, Trajectory._distance_unit, f.distance_unit))
+
+
     def save_netcdf(self, filename, force_overwrite=True):
         """Save trajectory in AMBER NetCDF format
 
@@ -1351,6 +1457,8 @@ _LoaderRegistry = {
     '.pdb': load_pdb,
     '.dcd': load_dcd,
     '.h5': load_hdf5,
+    '.crd': load_mdcrd,
+    '.mdcrd': load_mdcrd,
     #'.lh5': _load_legacy_hdf,
     '.binpos': load_binpos,
     '.ncdf': load_netcdf,
